=begin
**
** config.rb
** 27/SEP/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Configuration parsing functionality. An interface with the XML parser
** implementation.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'rexml/document'

module Core

  # This module contains all the classes and interfaces needed to read and
  # parse Dradis configuration files.
  # A ParserInterface is provided just in case we want to change the
  # configuration container in the future. In this version we are using an
  # XML file, but a YAML or a database could be used provided that 
  # ParserInterface is followed.
  module Config
    #--------------------------------------------------- parser interface
    
    # The ParserInterface should be implemented by all the different
    # configuration parsers (xml, yaml, etc.). Dradis and it's modules will
    # only access methods defined in this interface
    module ParserInterface
      # Given an option name (as a symbol) this function retrieves the value
      # stored in the configuration file for it.
      def get_option(key)  
        raise 'unimplemented!'
      end
      
      # Store the given +value+ under the +key+ in the config provider.
      def put_option(key, value)
        raise 'unimplemented!'
      end
      
      # Store the configuration settings in the backend provider.
      def save
        raise 'unimplemented!'
      end
    end
    
    #--------------------------------------------------- parser implementations
    
    # ParserInterface implementation to read configuration parameters from a
    # XML file.
    class XMLParser
      include ParserInterface
      # Returns :*xml* symbol. This method is used by the Factory to choose the
      # appropiate implementation for a given file.
      def XMLParser.get_type() return :xml; end

      # Read and parse the XML file. Stores the options in an internal Hash to
      # speed up the retrieving process.
      def initialize(file)
        @file = file
        fd = File.new(@file)
        @src = REXML::Document.new(fd)
        fd.close()
        @options = {}
        @modified = false
        @src.elements.each('dradis/option') do |element|
          @options[element.attributes['name'].to_sym] = element.attributes['value']
        end
      end
      
      # See ParserInterface#get_option
      def get_option(key)
        if @options.key?(key)
          @options[key]
        else
          raise ArgumentError.new('option not found in config file!')
        end
      end
      
      # See ParserInterface#put_option
      def put_option(key, value)
        if @options.key?(key)
          return if @options[key] == value
        end
        @options[key] = value
        @modified = true
      end
      
      # See ParserInterface#save
      def save
        return unless @modified
        @options.each do |name, value|
          elements = @src.get_elements("dradis/option[@name='#{name}']")
          if (elements.size.zero?)
            # a new element needs to be created
            @src.root.add_element( 'option', { 'name' => name, 'value' => value})
          else
            # update the existing element
            elements.first.attributes['value'] = value
          end
        end
        fd = File.new(@file, 'w')
        fmt = REXML::Formatters::Pretty.new(2)
        fmt.write( @src, fd )
        fd.close()
      end
    end
    
    #--------------------------------------------------- parser factory
    
    # Configuration Parser Factory class, used to get a valid configuration
    # parser for a given filename. The filename's extension is used to 
    # instantiate a valid ParserInterface implementation.
    class Factory
      @@factories = [XMLParser]
      
      # Given a configuration filename, this function returns a configuration
      # parser object that is able to read the values from the file.
      def Factory.from(file)
        type = File.extname(file).to_sym
        
        @@factories.each do |parser|
          parser.get_type() == type 
          return parser.new(file)
        end 
      end
    end
  end
end

# activeresource needs access to the $config straight away
$config = Core::Config::Factory.from('conf/dradis.xml')
