=begin
**
** controller.rb
** 03/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   One of the important pieces of the architecture, this file contains the 
** implementation of the Controller interface. The Controller connects the
** View to the Service layer.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core
  # Context is the provided implementation of the Controller interface.
  class Controller
    include Interfaces::Controller
    include Patterns::Observable
    add_event :exception    
    
    #===================================================== private methods
    private
    
    # Given a file (on the LOAD PATH), it loads, instantiates and verifies
    # that the file implements the Interfaces::ServiceProvider interface
    def load_provider(file)
      provider_name = File.basename(file, '.rb').camelize
      $logger.debug{ "Service provider file in: #{file}." }
      $logger.debug{ "\tClass #{provider_name}::Provider." }
      $logger.debug{ "\tLoading file..." }
      load file
      $logger.debug{ "\tLoaded." }
      $logger.debug{ "\tInstantiating Providers::#{provider_name}::Provider.."}
      provider_instance = nil
      begin
        provider_instance = eval("Providers::#{provider_name}::Provider").new(:controller => self)
      rescue
        $logger.error{ "Please ensure that your service provider in file #{file} defines a class Provider inside the Core::Providers::#{provider_name} module" }
        $logger.debug{ $! }
        next
      end
      $logger.debug{ "\tInstantiated."}
      
      if !provider_instance.class.included_modules.include?(Interfaces::ServiceProvider)
        raise "Please ensure that your service provider #{provider_name}::Provider [#{file}] includes the Interfaces::ServiceProvider module."
      end
      $logger.debug{ "\tProvider verified."}
      return provider_instance
    end

    # Find all the ruby files in a given path and load and instantiate the
    # service providers
    def find_providers(path)
      # don't go any further if the providers directory doesn't exist
      if not File.exists?(path)
        raise ServiceProvidersPathNotFoundException.new("Service Providers path does not exist: [#{path}]") 
      end
      
      # add a trailing slash
      path += '/' unless (path =~ /\/$/)
      $: << path
      provider_list = []
      
      $logger.info{ "Loading service providers from #{path} folder." }
      Dir.entries(path).each do |provider|
        next if File.extname(provider) != '.rb'
        begin
          provider_list << load_provider(provider)
        rescue
          $logger.error{ "There was an error loading the service provider at #{provider}: #{$!}" }
          $logger.debug{ $! }
        end
      end
      $logger.info{ "Service providers found: #{provider_list.collect do |p| p.class.to_s end.join(', ')}." }
      return provider_list
    end
    
    # This method uses the Interfaces::ServiceProvider interface to gather 
    # information about the different services provided by each provider.
    def find_services(providers)
      service_list = {}
      providers.each do |provider|
        services = provider.get_service_list()
        services.each do |service|
          service_list[service] = provider
        end
      end
      return service_list
    end

    # This is a two step process: 
    # * find the providers, and load them. 
    # * find the services each provider is offering
    def init_providers(path)
      # Find service provider files, instantiate them and verify they follow
      # the interface
      providers = find_providers(path)
      # For the list of providers found, find out the services they provide
      @services = find_services(providers)
    end    
    #===================================================== public methods
    public
    
    # See Interfaces::Controller#initialize
    def initialize(params)
      # init the View
      # verify that the parameter exists and is not nil
      if ( !params.key?(:view) || params.fetch(:view,nil).nil? )
        raise ArgumentError.new('You need to define a :view') 
      end
      
      @view = params[:view]
      # verify the view implements the Core::Interfaces::View interface
      if (!@view.class.included_modules.include?(Interfaces::View))
        raise ArgumentError.new('The :view must implement Core::Interfaces::View') 
      end
      @view.setup(self)

      sp_path = params.fetch(:service_providers, './core/serviceproviders' )
      init_providers( sp_path )
    end
    
    # See Interfaces::Controller#exec
    def exec() 
      # TODO: maybe this can be converted into a request_service(:view_exec) if
      # the view implemented the ServiceProvider interface.
      @view.run
    end
    
    # See Interfaces::Controller#request_service
    def request_service(key, *args)
      $logger.debug( "Service request: #{key}." )
      if !@services.key?(key)
        signal_event :exception, "The service [:#{key}] is not recognised."
        $logger.error { "The service [:#{key}] is not recognised." }
        return false
      end
      
      begin
        $logger.debug( "Service provider for #{key} is: #{@services[key].class}." )
        return @services[key].request_service(key, *args)
      rescue Errno::ECONNREFUSED
        signal_event :exception, 'Error while connecting. Is the server running?', $!
        $logger.error{$!}
      rescue DradisQuitApp
        # user requested end of session through 'quit' command
        # TODO: replace this for an event signal/notify pattern
        $logger.info{ 'User requested shutdown.' }
        @view.teardown        
        return '~'*60+"/dradis - #{VERSION::STRING}"
      rescue
        # if there is an exception requesting a service, signal it
        signal_event :exception, "[error] #{$!}", $!
        $logger.error { $! }
        $logger.debug {$@.join("\n\t")}
      end
      # upon failure, return false
      return false
    end
  end

end
