=begin
**
** interfaces.rb
** 30/JUL/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   One of the important pieces of the architecture, this file contains the 
** definition of the Controller, Service and View interfaces.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end



module Core
  module Interfaces
    
    #----------------------------------------------------------------------
    #----------------------------------------------------- Model
    #----------------------------------------------------------------------

    # Interface for the Model object. The model is responsible for storing all
    # the relevant information in the server's knowledge base.
    module Model
      # Returns the revision number of the local version of the KnowledgeBase.
      def revision() raise 'unimplemented!' end
      # Checks if there is a new version of the KnowledgeBase in the server. If 
      # a new revision exist is downloaded and +true+ is returned, otherwise 
      # +false+ is returned.
      def new_revision?() raise 'unimplemented!' end
      
      # Pull down a copy of the KnowledgeBase
      def refresh() raise 'unimplemented!' end    
        
      # Returns an array that contains the children of a specific node. If parent_id
      # is +nil+, the root nodes are returned. 
      def node_children(parent_id) raise 'unimplemented!' end
      # Returns the list of all the Note categories defined in the server
      def categories() raise 'unimplemented!' end
      
      # dradis2 functionality
      def node_add(parent_id, type_id, label) raise 'unimplemented!' end
      #def del_node(id) raise 'unimplemented!' end
      def node_update(node_id, new_parent_id, new_type_id, new_label) raise 'unimplemented!' end

      def note_add(note_id, author, category_id, text) raise 'unimplemented!' end
      #def note_del(note_id) raise 'unimplemented!' end
      def note_update(note_id, new_author, new_category_id, new_text) raise 'unimplemented!' end

      #def node_find(node_id) raise 'unimplemented!' end 
      def note_find(note_id) raise 'unimplemented!' end 
      def category_find(category_id) raise 'unimplemented!' end      
    end
  
    
    #----------------------------------------------------------------------
    #----------------------------------------------------- View
    #----------------------------------------------------------------------
    
    # In order to create a new View, this simple interface must be followed.
    module View
      include Patterns::Observer
            
      # TODO: fix the constructor implementation issue Qt vs. WxWidgets
      #def initialize(argv) raise 'unimplemented!' end
      def setup(new_controller) 
        raise NotImplementedError
      end
    
      # When the service layer raises a Core::DradisQuitApp exception, the 
      # controller will catch it and notify the view by invoking this method.
      def teardown() 
        raise NotImplementedError
      end

      # transfer control of the execution flow to the View
      def run() 
        raise NotImplementedError
      end
    end


    #----------------------------------------------------------------------
    #----------------------------------------------------- Controller
    #----------------------------------------------------------------------
    
    # The Controller will connect the View with the different ServiceProviders. 
    # This interface provides the methods available to be called by both the 
    # View and the Providers.
    module Controller
      # constructor method, it
      def initialize(params)
        raise NotImplementedError
      end
      
      # The view or the providers do request a service from the controller 
      # through this function
      def request_service(service_key, *args)
        raise NotImplementedError
      end

      # The main process uses this call to transfer the execution flow control
      # to the controller.
      def exec()
        raise NoteImplementedError
      end
    end
    
    
    #----------------------------------------------------------------------
    #----------------------------------------------------- Service Provider
    #----------------------------------------------------------------------
    
    # The ServiceProviders are components that expose 'services' that other
    # components of the application can use. Examples of providers:
    #
    # * Interfaceerator: provides services to convert XML strings to graphical
    # interface elements (through the classes in Core::Model::Support)
    # * Multiverse: provides services to access the information stored in the
    # different universes loaded
    # * Commands: provides services to access user defined commands
    #
    # ServiceProvider implementations are located in the Core::ServiceProvider
    # module
    module ServiceProvider
      
      # Service providers must take a variety of configuration parameters, for
      # instance a :controller will point to the instantiatin controller and
      # can be used by the provider to request services from other providers
      def initialize(params={}) raise 'unimplemented!' end
        
      # Retrieve a list of all the services provided by this provider. The list
      # consists of an array of service_key elements. This will be used by the
      # Controller when a request for service is made to locate the right 
      # ServiceProvider to supply it.
      def get_service_list()
        services = []
        self.public_methods.each do |method|
          next unless method =~ /^service_/
          services << $'.to_sym
        end
        return services
      end 
        
      # Retrieve information of a specific service, this information includes
      # a description and probably syntax specifications so the Controller can
      # perform input validation before the request is transfered to the 
      # provider
      def get_service_info(service_key) raise 'unimplemented!' end 
        
      # Request a service to this provider. A service key and arguments for the
      # service are accepted.
      def request_service(key, *args)
        method = "service_#{key}"
        self.send(method, *args)
      end
    end #ServiceProvider

    #----------------------------------------------------------------------
    #----------------------------------------------------- Extensions
    #----------------------------------------------------------------------

    # Provides the interface that extensions need to follow. The interface only 
    # requires from the extension the ability to declare the actions that it 
    # supports Extension#commands) and to be able to perform the action 
    # once the user has decided to do so Extension#run.
    #
    # The details of the module should be provided in a Hash similar to this one:
    #  INFO = { 
    #    :commands => {
    #      'command #1' => {}, #options for the first command
    #      'command #2' => {}, #options for the second command
    #      #...
    #    }
    #  }
    #
    # The :*commands* key contains a Hash with all the information regarding the
    # different commands provided by the extension. This keys for this hash
    # must be the command labels (+help+, +debug+, etc.) and the values must 
    # specify the options for each commands. For an example of this please see
    # Extension#commands.
    #
    # Implementing classes could add further restrictions and features to the
    # above structure but the :*commands* key should be retained..
    module Extension
      # Should return a Hash with the commands provided by the module. an example
      # of the syntax that the hash elements should follow is:
      # 
      #  'ls' => { 
      #    :desc => 'lists entries under the directory specified as argument', 
      #    :syntax => [ 
      #      {
      #        :required => true,
      #        :label=>'path', 
      #        :regexp=> /.+/
      #      }
      #    ]
      #  }
      #        
      # Because the framework will handle input validation, syntax checking,
      # help requests, etc. some information should be provided by the implementing
      # modules to do so. In a nutshell the description of the structure above:
      # [+ls+] is the name of the action. when the user enters +ls+ the method
      #        *ls* of the implementing module will be called.
      # [:*desc*] is the description of the nature of the action.
      # [:*syntax*] is an array of parameters accepted by the method. Each
      #             parameter is described as a Hash that contains three keys:
      #             [:*required*] states whether the parameter is required or
      #                           optional
      #             [:*label*] is the name of the parameter. This name will be
      #                        displayed if the user request help for the action
      #             [:*regexp*] the regular expresion that the parameter needs
      #                         to match
      def commands() 
        raise NotImplementedError
      end

      # Once the parser has decided the action that the user wants to perform,
      # this method is called.
      #
      # The first argument (_cmd_) is the action requested by the user. The
      # second is an array of the arguments provided by the user.
      #
      # No details for the implementation are provided here, but as a general
      # guideline this method should call to one of the methods of the module
      # depending on the value of _cmd_.
      def run(cmd, *args) 
        raise NotImplementedError
      end
    end #Extension

  end #Interfaces
end #Core



