=begin
**
** patterns.rb
** 28/MAY/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Interfaces and modules used to implement different design patterns 
** throughout the other components of dradis.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core
  
  # Interfaces and modules used to implement different design patterns 
  # throughout the other components of dradis.  
  module Patterns  
    
    # Implementation of the Observer pattern, this module defines a single
    # method: +notify+. Observable objects will invoke this method of their
    # observers every time an event to which the observer is registered is
    # signaled. 
    module Observer
      
      # This method is invoked by the Observable object if the Observer is 
      # associated with the +event+ being signaled.
      # The default implementation provided by this module raises an exception
      # Observer classes are expected to override this behaviour.
      def notify(event, *args)
        raise "#{self.class.name} (#{self.inspect}) does not implement a valid notify() function. Event notified: #{event} (#{args})"
      end
    end
        
    #Implementation of the Observable pattern. Classes including this module 
    #will be able to signal events. This event will be notified to the list
    #of registered _observers_ for this class. This is an example on how to use
    #this pattern:
    #
    # class Controller
    #   include Observable
    #   add_event :before_load, :after_load
    #
    #   def load
    #     signal_event :before_load
    #     print 'Loding... '
    #     sleep 2
    #     puts 'done.'
    #     signal_event :after_load
    #   end
    # end
    #
    # class Service
    #   include Observer
    # end
    #
    # c = Controller.new 
    # c.add_observer :before_load, Service.new  # -> Registered observers:    
    #                                           # -> {:before_load=>[<Service:0x24aa4>]}
    #
    # c.add_observer :after_load, Service.new   # -> Registered observers:
    #                                           # -> {:before_load=>[<Service:0x24aa4>], :after_load=>[<Service:0x247d4>]}
    # 
    # c.load                                    # -> Service (<Service:0x24aa4>) notified of :before_load
    #                                           # -> Loding... done.
    #                                           # -> Service (<Service:0x247d4>) notified of :after_load    
    #
    module Observable
      # TODO: handle logging in a better way than with 'puts'
      private
      def self.included(base)
        base.extend ClassMethods
      end
      public
      # Observers of any object are registered using this instance method. The
      # +obj+ must include the Observer module. A symbol for one of the events
      # registered for this class (See ClassMethods#add_event and 
      # ClassMethods#get_events) should be provided in +event+.
      def add_observer(event, obj)
        # check that the event +obj+ is trying to subscribe is provided by this class.
        if not self.class.get_events.include?(event)
          raise NameError.new(
            "The event :#{event} is not recognized by the class #{self.class.name}. " +
            "Use #{self.class.name}.get_events() to get a list of valid events for the class."
            )
        end

        # initialize this object's instance variables if this has not been done 
        # before
        @observers = {} if not defined? @observers
        @observers[event] = [] if not @observers.key?(event)

        # register the subscriber for the +event+
        if @observers[event].include?(obj)
          # TODO: show a warning, the observer was previously added to this event
          puts "Warning, #{obj.inspect} is already and observer for :#{event} in this object."
        else
          @observers[event] << obj 
        end

        puts "Registered observers:\n\t#{@observers.inspect}"
      end
      
      # Get the list of current Observers of this instance. If no +event+ is
      # provided, the full hash of Observers of this object is returned.
      def get_observers(event=nil)
        @observers = {} if not defined? @observers
        event.nil? ? @observers : @observers.fetch(event, nil)
      end

      protected
      # This method is used by objects that are instances of classes that 
      # implement the Observable pattern to notify to the different 
      # Observers registered that an event has just occured.
      #
      # * +event+ the symbol that represents the event being signaled. See 
      #   ClassMethods#add_event for details.
      # * +args+ a list of parameters associated with the event. The default 
      #   implementation prepends the a reference of the object signaling the 
      #   event so the Observers can identify it.
      def signal_event(event, *args)
        @observers = {} if not defined? @observers
        @observers.fetch(event, []).each do |observer|
          args.unshift(self)
          observer.notify event, *args
        end
        return nil
      end

      # This methods will be used to extend the class that includes the 
      # Observable module.
      module ClassMethods 
        # This method is used inside the including classes to register the list
        # of events supported by the class.
        # * +args+ is a list of one or more symbols (i.e. :*before_load*, 
        #   :*after_load*)
        def add_event(*args)
          puts "event(s) registered: #{args.inspect}"
          @events = args
        end

        # This method returns the list of events supported by an including class.
        def get_events
          @events
        end

      end
    end
  end
end
