=begin
**
** command_launcher.rb
** 06/AUG/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   This service provider will enable the user to execute commands from 
** the console. Commands can be defined by the users in the ./extensions/
** directory.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core
  module Providers
    module CommandLauncher
      # Interfaceerator is the module that translates from XML into property
      # objects. These can be used by the View to present the user with the 
      # options as the creator defines them in universe.xml.  
      class Provider < Extensions::Simple
        include Interfaces::ServiceProvider
        attr_accessor :dispatchers
        attr :cmd_found
        
        #===================================================== private methods
        private

        # Returns a Hash of commands supported by this Dispatcher. See 
        # Dispatcher#commands for output format.
        # 
        # The current list of supported commands for this Dispatcher is:-
        # [quit] Exits the application
        # [help] Shows the help of all the commands available
        INFO = { 
          :commands => {
            'reload' => {
              :desc => 'reload the modules from the filesystem',
              :syntax => []#no params
            },
            'help' => { 
              :desc => 'shows the help of all the commands available', 
              :syntax => []#no params
            },
            # TODO: does it make sense to have a 'quit' command here? wouldn't it
            # be better if the Parser of the View has some 'dispatching' 
            # functionality of this kind?        
            'quit' => { 
              :desc => 'exits the application', 
              :syntax => []#no params
            }
          }
        }

        def reload(*args)
          $logger.debug{ "\t\tLoading external modules from extensions/ folder." }
          self.dispatchers = [self]
          Dir.entries('extensions/').each do |dispatcher|
            next if File.extname(dispatcher) != '.rb'
            klass = File.basename(dispatcher, '.rb').camelize
            begin
              load "extensions/#{dispatcher}"
              self.dispatchers << eval("Extensions::#{klass}").new(:controller => @controller)
            rescue Exception => e
              $logger.error{ "\t\tError loading #{dispatcher} extension: #{e.message}" }
              $logger.debug{ e }
            end
          end
           
          $logger.debug{ "\t\tExternal modules found: #{self.dispatchers.collect do |d| d.class.to_s end.join(', ')}" }
        end


        #
        # For a given command (_cmd_), this method walks through the registered
        # dispatchers to see if anyone is able to run the command. If a valid 
        # dispatcher is found, the _run_ method for this dispatcher is called 
        # passing along the command name and parameters entered by the user.
        #
        def dispatch(cmd, *args)
          @cmd_found = false
          out = "command <b>#{cmd}</b> not implemented"
          self.dispatchers.each do |d|
            next if !d.respond_to?('commands') ||
                        d.commands == nil || 
                        d.commands.size == 0
        
            if (d.commands.has_key? cmd)
              # TODO: this is a fix to maintain compatibility with old 
              # extensions
              args.unshift(cmd)
              out = d.run(cmd, *args)
              @cmd_found = true
            end
            break if @cmd_found
          end
      
          if out.instance_of?(Array)
            return out.join("\n")
          else
            return out
          end      
        end        
      
        #----------------------------------------------------- command impl.
        def quit(*args)
          raise DradisQuitApp
        end

        # Handle +help+ requests. A list of available commands are returned.
        def help(*args)
          commands = ['List of registered commands']
          commands << '---------------------------'
          self.dispatchers.each do |d|
            next if !d.respond_to?('commands') ||
                        d.commands == nil || 
                        d.commands.size == 0
            d.commands.each do |cmd, opts| commands << "#{cmd}:\t#{opts[:desc]}" end
          end
          commands
        end    

        # Splits a string into arguments but taking into account single and 
        # double quotes.
        # This code existed in dradis 1.x in the controller class. See:
        # http://dradis.svn.sourceforge.net/viewvc/dradis/client/branches/RB-1.2/core/controller.rb?view=markup
        def smart_split(line)
          # parse the commandline taking quotes and double quotes in account
          args = line.split
          # will keep args after parsing
          final_args = []
          counter = 0      
          # loop through whole array
          while counter < args.length
            for i1 in counter..args.length-1
              final_args << args[i1]
              counter = counter + 1
              # we see if element starts with " or '
              if args[i1][0,1] == '"' or args[i1][0,1] == '\''
                # value remember if we found a " or a '
                value = args[i1][0,1]
                # lets just check that this argument does not end in a quote
                if args[i1][-1,1] == value
                  final_args << final_args.pop[1...-1]
                  break
                else
                  final_args << final_args.pop[1..-1]
                end
                # now loop through the subsequent elements
                for i2 in i1+1..args.length-1
                  # pad with space
                  final_args.last << ' '
                  # append element
                  final_args.last << args[i2]
                  counter = counter + 1
                  # check if element ends with +value+
                  if args[i2][-1,1] == value
                    # amend and break if this is the case
                    final_args << final_args.pop[0...-1]
                    break
                  end
                end
                break
              end
            end
          end
          return final_args
        end
      
        #===================================================== public methods
        public

        def initialize(params={})
          @controller = params[:controller] 
          reload()
        end

        #----------------------------------------------------------------------
        #----------------------------------------------------- services
        #----------------------------------------------------------------------
        def service_parse_command(*args)
          if (args.size < 1)
            raise ServiceRequestInvalidArgumentError.new('Invalid parameters. Required parameters: command')
          end        

          params = smart_split(args[0])
          cmd = params.shift 

          # If the user does Ctrl+C, a nil value is passed 
          return '' if cmd.nil?

          # step 1: cycle through the external modules
          out = dispatch(cmd, *params)
          return out if @cmd_found

          # step 2: cycle through the internal services
          # the following call never raises and exception
          @controller.request_service(cmd.to_sym, *params)

          return 'Service request executed successfuly'
        end
        
        def service_echo(*args)
          puts args.join(' ')
        end
 
        
      end
    end
  end
end
