=begin
**
** interfaces.rb
** 02/JUL/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Contains the interfaces that modules used by the multiverse framework need
**  need to follow.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core
  module Providers
    module Multiverse
      # This module contains the interfaces of the different components of the 
      # multiverse framework
      module Interfaces    
        # The Parser interface requires from it's including classes to implement
        # a parse method to parse sources.
        module Parser  
          def parse(source) 
            raise NotImplementedError
          end 
        end
      
        module Universe
          attr_accessor :nodes # array of nodes
          attr_accessor :meta # hash of meta information
          attr_accessor :path # path to the universe base dir
        end
      
        module Builder
          attr_accessor :multiverse # array of universes
        end
      
        module Parsers
          # Each section of the universe XML file will be parsed by a class that
          # follows this interface.
          module SectionParser
            # This method is invoked before the document is parsed and can be
            # used by the section parser to verify that specific tags or 
            # attributes are present.
            def check(document)
              raise NotImplementedError
            end
            
            # This method should parse the section and return the relevant ruby
            # objects.
            def parse(document, universe) 
              raise NotImplementedError
            end
          end
        end
        
        # Each property populator will be able to automatically populate a 
        # property on behalf of the user.
        module PropertyPopulator
          # Invoke the populator. This method expects two parameters:
          # * +:node_id+: The id of the node whose property we are populating
          # * +:properties+: An array of Core::Model::Support::Property 
          # objects that represents the current set of properties for the node.
          # The method returns an Array of properties that have been modified,
          # an empty array if none have been.
          def invoke(params={}) 
            if ([:node_id, :properties, :controller] & params.keys).size != 3
              raise "Please invoke this populator passing at least the following options: :node_id, :properties, :controller"
            end
            
            on_invoke(params)
          end
        end
      end    
    end
  end
end
