=begin
**
** parser.rb
** 02/JUL/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'rexml/document'

module Core
  module Providers
    module Multiverse
    
      class Parser
        include Interfaces::Parser
      
        #===================================================== private methods
        private

        #----------------------------------------------------- validations
        def check_xml_source(source)
          if (source.class != Hash)
            raise Core::Exceptions::Multiverse::UniverseParsingException.new('Please provide a Hash containing one of the following keys: :file, :string') 
          end
        
          xmlsource = nil
          # check if parsing a file
          filename = source.fetch(:file, nil)
          if filename
            raise FileNotFound(filename) if not File.exists?(filename)
            xmlsource = File.new(filename)
            @directory = File.dirname(filename)
          else
            # check if it is a string
            xmlsource = source.fetch(:string, nil)
          end
          raise Core::Exceptions::Multiverse::UniverseParsingException.new('No XML source was given.') if xmlsource.nil?
        
          return xmlsource
        end
      
        def check_xml_structure(document)
          # check the document root
          if document.root.name != 'universe'
            raise Core::Exceptions::Multiverse::UniverseParsingException.new('<universe> tag has to be the root element.')
          end

          # the +meta+ section is mandatory
          if (document.root.elements['meta'].nil?)
            raise Core::Exceptions::Multiverse::UniverseParsingException.new('No <meta> tag was found.') 
          end
        end
      
        def parse_section(document, universe, section)
          # 1.- find the parser
          file = "multiverse/parsers/#{section}_section_parser.rb"
          begin
            load file
          rescue LoadError
            # if we can't identify the section, maybe this universe was created
            # for a newer version of dradis. do nothing, just record a debug
            # message.
            $logger.debug { 
              "\t\t\tFile #{file} not found (was required for universe XML parsing)." 
            }
            return
          end
          # See Rails' > activesupport > lib > active_support > inflector.rb
          klass = File.basename(file, '.rb').gsub(/(^|_)(.)/) { $2.upcase }

          # 2.- instantiate
          parser = eval('Parsers::'+klass).new

          # 3.- check
          begin
            parser.check(document)
          rescue NotImplementedError
            $logger.debug{ "\t\t\tThe #{klass} does not implement the +check()+ method. Aborting." }
            return
          end

          # 4.- parse
          begin
            parser.parse(document, universe)
          rescue NotImplementedError
            $logger.debug{ "\t\t\tThe #{klass} does not implement the +parse()+ method. Aborting." }
            return
          end

        end
      
        #===================================================== public methods
        public

        # Parse the contents of the XML file or string and return a Universe 
        # object that contains the elements defined in the file.
        # +xmlsource+ is a hash, if it contains a :file element, the XML file
        # is parsed, otherwise, a :string element containing the XML source is 
        # expected.
        # If no :file or :string elements are passed an exception will be 
        # raised.
        def parse(source = {})
          xmlsource = check_xml_source(source)
          begin
            doc = REXML::Document.new(xmlsource)
          rescue REXML::ParseException => e # re-raise exception
            raise Core::Exceptions::Multiverse::UniverseParsingException.new(e)
          end
          check_xml_structure(doc)

          u = Universe.new
          if @directory
            u.path = @directory
          end
          doc.root.each_element do |section|
            parse_section( doc, u, section.name )
          end
          return u
        end
      end
    end
  end
end
