# = Synopsys
#
# This module is the main entry point of the application. Within it, the user
# interface is instanced and launched.
#
# = Author
# 
# Daniel Martin Gomez <etd[-at-]nomejortu.com>
#
# = Usage
# 
#   ruby dradis.rb [ -h | -help ] [ -v | --version ] [ -c | -g ]
#   
#   -h | --help: display application synopsys, author and usage information.
#   -v | --version: display version informaiton.
#   -c: choose comand line interface (default).
#   -g: start graphical user interface (same as -w).
#
#   other valid options: 
#   -w: choose a GUI based on wxWidgets (requires wxruby gem).
#   -q: choose a GUI based on Qt4 (requires libqt4-ruby bindings) [deprecated].
# 
# = License
# 
# This file may be used under the terms of the GNU General Public
# License version 2.0 as published by the Free Software Foundation
# and appearing in the file LICENSE.txt included in the packaging of
# this file.
#
# Copyright: Daniel Martin Gomez <etd[-at-]nomejortu.com>
#

# standard library modules
require 'optparse'
require 'rdoc/usage'
require 'logger'

# core dradis modules
require 'core'

if $0 == __FILE__
  ui = :cli
  
  opts = OptionParser.new
  opts.on('-h', '--help') {RDoc::usage}
  opts.on('-v', '--version') { Core::VERSION::show }
  opts.on('-c') {ui = :cli}
  opts.on('-g') {ui = :wx}
  opts.on('-q') {ui = :qt}
  opts.on('-w') {ui = :wx}
  opts.parse(ARGV) rescue RDoc::usage(0, 'usage')

  $logger = Logger.new(STDOUT)
  # use the standard Formatter instead of the rails Silenced one
  $logger.formatter = Logger::Formatter.new
  #$logger.level = Logger::DEBUG
  $logger.level = Logger::FATAL
  $logger.datetime_format = '%H:%M:%S'
  
  begin
    # The central component of dradis is the controller, it talks to the 
    # View (which presents info to the user and gathers action requests)
    # and to the Service Providers (which are able to perform the requested 
    # actions)
    
    # First select a View
    case ui
    when :qt #Qt application
      $logger.debug{'starting Qt interface'}
      require 'ui/qt/gui'
      ui = Ui::Qt4::DradisGui.new(ARGV)
    when :wx # wXWidgets application
      $logger.debug{'starting wxWidgets interface'}
      require 'ui/wx'
      ui = Ui::WxWidgets::DradisGui.new
    when :cli #Console application
      $logger.debug{'starting command line interface'}
      require 'ui/console'
      ui = Ui::Console::CLI.new()
    end

    # and then the Controller
    controller = Controller.new(:view => ui)
    controller.exec
  
  rescue LoadError
    $logger.error('Please try an alternative client interface. Use -h for options')
  ensure
    $config.save
    $logger.close  
  end
end
