=begin
**
** import.rb
** 11 February 2009
** Siebert Lubbe siebertlubbe [ at ] googlemail [ dot ] com
**
** Desc:
** Import data into dradis from various sources
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end


module Extensions
  # The module presents methods to import from external data sources into Dradis
  # Actions:
  # [+nmap+] Import data from an nmap result file
  class Import < Extensions::Namespace

    # used parsing XML output
    require 'rexml/document'
    include REXML

    #----------------------------------------------------- private methods
    private

    #----------------------------------------------------- Dispatcher implementation
    INFO = {
      :namespace => 'import',
      :description => 'import external resources to dradis',
      :commands => {
        'nmap' => {
          :desc => 'Import nmap results file',
          :syntax => [
          {
            :required => true,
            :label => 'file name',
            :desc => 'The file that contains the nmap output',
            :regexp => /.*/
          },
          {
            :required => true,
            :label => 'nmap output format',
            :regexp => /normal|grepable|xml/,
            :desc => 'Either normal or grepable'
          },
          {
            :required => false,
            :label => 'parent id',
            :regexp => /\d*/,
            :desc => 'The parent node under which the new nodes will be created'
          }
          ]
        },
        'textfile' => {
          :desc => 'Import the content of a text file as a note',
          :syntax => [
            {
              :required => true,
              :label => 'filename',
              :desc => 'The text file that contains the note',
              :regexp => /.*/
            },
            {
              :required => true,
              :label => 'node id',
              :regexp => /^\d*$/,
              :desc => 'The node that the note should be added to'
            },
            {
              :required => true,
              :label => 'category',
              :regexp => /.*/,
              :desc => 'The category to be stored with this note'
            }
          ]
        }
      }
    }

    # Implements the nmap results import functionality. See above for parameters
    def nmap(*args)
      return "#{args[1]} is not a valid file" if !File.exists?(args[1])

      # allow 0 as a parent id but turn it into a nil
      args[3] = (args[3] == '0' || args[3] == nil) ? nil : args[3].to_i
      # check if the parent id exist if it was passed
      return "Parent node with id #{args[3].to_s} does not exist" if args[3] != nil &&
        @controller.request_service(:model_find_node, args[3]).label == "error, this node does not exist in the server!!"

      # open file and read contents into variable
      file_handle = File.new(args[1])
      file_content = file_handle.readlines

      case args[2]
      when "grepable"
        hosts = parse_grepable_output(file_content)
      when "normal"
        hosts = parse_normal_output(file_content)
      when "xml"
        hosts = parse_xml_output(file_content)
      end


      # get the "nmap output" category instance or create it if it does not exist
      category = @controller.request_service(:model_find_category_by_name, "Nmap output") || @controller.request_service(:model_add_category, "Nmap output")

      port_notes_to_add = {}
      hosts.each do |host, detail|
        host_node = @controller.request_service(:model_add_node, args[3], nil, host)
        # refresh the local copy of the knowledge cache to get the id of the new node in the knowledge cache
        @controller.request_service(:model_knowledge_refresh)
        # add the nmap output for the host as notes to the node
        @controller.request_service(:model_add_note, host_node.id, "Nmap", category.id, detail[:notes])
        detail[:ports].each do |port, detail|
          port_node = @controller.request_service(:model_add_node, host_node.id, nil, "#{port}/#{detail[:protocol]}")
          # we need the id of the added
          port_notes_to_add[port_node.id] = "State: #{detail[:state]}, Service: #{detail[:service]}"
        end
      end

      # refresh the local copy of the knowledge cache to get the ids of the new nodes in the knowledge cache
      @controller.request_service(:model_knowledge_refresh)
      port_notes_to_add.each do |port_node_id, note|
        @controller.request_service(:model_add_note, port_node_id, "Nmap", category.id, note)
      end

      return "#{args[1]} was successfully imported"
      #return hosts.inspect

    end # def nmap
    
    # parse nmap output that is in normal format
    def parse_normal_output(file_content)
      # initiate some variables to be used
      hosts = {}
      host = ""
      leading_line = ""
      # loop through the contents of the file and populate the hosts hash variable
      file_content.each do |line|
        case line.strip
        when /^Starting Nmap/
          leading_line = line
        when /^Interesting ports on (.*):/
          host = $1
          hosts[host] = {:notes => leading_line+line}
          hosts[host][:ports] = {}
        when /^(\d*)\/(tcp|udp) *(open|filtered|closed|unfiltered) *(.*)/
          hosts[host][:ports][$1] = {:protocol => $2, :state => $3, :service => $4}
          hosts[host][:notes] << line
        when /^PORT *STATE *SERVICE/
          hosts[host][:notes] << line
        when /^Not shown/
          hosts[host][:notes] << line
        else
          next
        end
      end
      return hosts
    end

    # parse nmap output that is in grepable format
    def parse_grepable_output(file_content)
      # initiate some variables to be used
      hosts = {}
      host = ""
      leading_line = ""
      # loop through the contents of the file and populate the hosts hash variable
      file_content.each do |line|
        case line.strip
        when /^# Nmap.*scan/
          leading_line << line
        when /^# Ports scanned/
          leading_line << line
        when /^Host:(.*)Ports:/
          host = $1
          hosts[host] = {:notes => leading_line+line}
          hosts[host][:ports] = {}
          hosts[host][:notes] << line
          ports = line.split(", ")
          ports.each do |port|
            port =~ /(\d*)\/(open|filtered|closed|unfiltered)\/(tcp|upd)\/.*\/(.*)\/.*\/(.*)\//
            hosts[host][:ports][$1] = {:protocol => $3, :state => $2, :service => $4+$5}
          end
        else
          next
        end
      end
      return hosts
    end

    def parse_xml_output(content)
      # initiate some variables to be used
      hosts = {}
      xml = Document.new(content.to_s)
      
      # loop through all hosts
      XPath.match(xml[6],"//host").each do |host|
        hostnodename = host.elements['address'].attributes['addr']
        if (!host.elements['hostnames'].elements.empty?)
         hostnodename << '('
         host.elements['hostnames'].elements.to_a.collect do |hostname|
           hostname.attributes['name']
         end.join(",")
         hostnodename << ')'
        end

        hosts[hostnodename] = {:notes => host.to_s}
        hosts[hostnodename][:ports] = {}

        XPath.match(host.elements['ports'], "port").each do |port|
          hosts[hostnodename][:ports][port.attributes['portid']] = {
            :protocol => port.attributes['protocol'],
            :state => port.attributes['state'],
            :service => port.elements['service'].attributes['name']
          }
        end
      end
      
      return hosts
    end

    def textfile(*args)
      file_name = args[1]
      node_id = args[2]
      category_name = args[3]

      # check if the text file exists on the file system
      return "#{file_name} is not a valid file" if !File.exists?(file_name)

      # check if the parent id exist if it was passed
      puts @controller.request_service(:model_find_node, args[2]).inspect
      return "Node with id #{node_id.to_s} does not exist" if node_id == nil ||
        @controller.request_service(:model_find_node, node_id.to_i).label == "error, this node does not exist in the server!!"

      return "There is no category with the name '#{category_name}'" if !(category = @controller.request_service(:model_find_category_by_name, category_name))
      
      # open file and read contents into variable
      file_handle = File.new(file_name)
      file_content = file_handle.read

      # import the file contents to the datastore
      if @controller.request_service(:model_add_note, node_id, "Textfile", category.id, file_content)
        return "The text file was successfully imported"
      else
        return "An error occured while importing the text file"
      end
    end

  end
end
