=begin
**
** universe_parser_test.rb
** 02/JUL/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   TestCase for the universe Parser, examples of tests include: the parses
** identifies all the sections of a universe.xml, a malformed XML doesn't crash
** dradis, etc.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'logger'
require 'test/unit'

$: <<  File.dirname(__FILE__) + '/../'
require 'core/exception'
require 'core/patterns'
require 'core/interfaces'
$: <<  File.dirname(__FILE__) + '/../core/serviceproviders/'
require 'multiverse'

include Core
include Exceptions

class UniverseParserTest < Test::Unit::TestCase
  def setup
    @parser = Core::Providers::Multiverse::Parser.new
    $logger = Logger.new(STDOUT)
    $logger.level = Logger::DEBUG
    $logger.level = Logger::FATAL
    $logger.datetime_format = '%H:%M:%S'
  end
  
  # Check that the parser identifies all the nodes in the universe definition 
  # file
  def test_node_count
    file = File.dirname(__FILE__) + '/../multiverse/networking/universe.xml'
    universe = @parser.parse( :file => file )
        
    assert_equal 4, universe.nodes.size
  end
  
  # Check that the universe contains the bare minimun meta info. requirements 
  # in order for the universe to be usable:
  # * name
  # * uid
  def test_meta_basics
    # check that meta info contains name and uid, and that both are valid.
    # checking the uniqueness of uid is done in the multiverse test case
    xml_ok =<<EOXML 
    <universe>
      <meta>
        <name>Networking</name>
        <uid>baa4f8f55d68308fdbbed8c2f1eccfc3640f0cf9</uid>
      </meta>
    </universe>
EOXML
    xml_ko =<<EOXML 
    <universe>
      <meta>
        <uid></uid>
        <creator name="etd" email="etd[ at ]nomejortu- dot -com" />
        <url>http://dradis.nomejortu.com/</url>
      </meta>
    </universe>
EOXML

    assert_nothing_raised do
      universe = @parser.parse( :string => xml_ok )
    end
    
    assert_raises(Multiverse::UniverseParsingException) do
      universe = @parser.parse( :string => xml_ko )
    end
  end
  
  # Check that all the elements of the meta section are properly parsed
  def test_meta_full
    xml_ok =<<EOXML 
    <universe>
      <meta>
        <name>Networking</name>
        <version>1.0</version>
        <namespace>net</namespace>
        <uid>baa4f8f55d68308fdbbed8c2f1eccfc3640f0cf9</uid>
        <creator name="etd" email="etd[ at ]nomejortu- dot -com" />
        <url>http://dradis.nomejortu.com/</url>
      </meta>
    </universe>
EOXML
    universe = @parser.parse( :string => xml_ok )
    assert_equal 6, universe.meta.keys.size
    assert_equal 'Networking', universe.meta[:name]
    assert_equal '1.0', universe.meta[:version]
    assert_equal 'net', universe.meta[:namespace]
    assert_equal 'baa4f8f55d68308fdbbed8c2f1eccfc3640f0cf9', universe.meta[:uid]
    assert_equal 'etd', universe.meta[:creator][:name]
    assert_equal 'etd[ at ]nomejortu- dot -com', universe.meta[:creator][:email]
    assert_equal 'http://dradis.nomejortu.com/', universe.meta[:url]
  end
  
  # Check that the +creator+ tag is properly parsed, both name and email are
  # optional attributes
  def test_meta_creator
    xml_ok1 =<<EOXML 
    <universe>
      <meta>
        <name>Networking</name>
        <uid>baa4f8f55d68308fdbbed8c2f1eccfc3640f0cf9</uid>
        <creator name="etd" />
      </meta>
    </universe>
EOXML
    xml_ok2 =<<EOXML 
    <universe>
      <meta>
        <name>Networking</name>
        <uid>baa4f8f55d68308fdbbed8c2f1eccfc3640f0cf9</uid>
        <creator email="etd[ at ]nomejortu- dot -com" />
      </meta>
    </universe>
EOXML
    xml_ko =<<EOXML 
    <universe>
      <meta>
        <name>Networking</name>
        <uid>baa4f8f55d68308fdbbed8c2f1eccfc3640f0cf9</uid>
        <creator />
      </meta>
    </universe>
EOXML
    universe1 = @parser.parse( :string => xml_ok1 )
    assert_equal 3, universe1.meta.keys.size
    assert_equal 'etd', universe1.meta[:creator][:name]
    assert_nil universe1.meta[:creator].fetch(:email, nil)

    universe2 = @parser.parse( :string => xml_ok2 )
    assert_equal 3, universe2.meta.keys.size
    assert_equal 'etd[ at ]nomejortu- dot -com', universe2.meta[:creator][:email]
    assert_nil universe2.meta[:creator].fetch(:name, nil)

    assert_raises(Multiverse::UniverseParsingException) do
      universe3 = @parser.parse( :string => xml_ko )
    end    
  end

  # Check that exceptions caused by malformed XML are properly captured by 
  # the parser
  def test_malformed
    xml_ko =<<EOXML 
    <universe>
      <meta
        <cr---eator />
      </meta>
      //-->
    </universe>
EOXML
    assert_raises(Multiverse::UniverseParsingException) do
      universe = @parser.parse( :string => xml_ko )
    end
  end
  
  # Check that sections that are not understood by the framework are not 
  # parsed and do not raise exceptions
  def test_unknow_section
    xml_ok =<<EOXML 
    <universe>
      <meta>
        <name>Networking</name>
        <uid>baa4f8f55d68308fdbbed8c2f1eccfc3640f0cf9</uid>
        <creator email="etd[ at ]nomejortu- dot -com" />
      </meta>
      <etdsoft>
        <!-- this section must be ignored by the parser -->
      </etdsoft>
      <nodes>
        <node type="1" name="Host" />
        <node type="2" name="Protocol" />
      </nodes>      
    </universe>
EOXML
    universe = nil
    assert_nothing_raised do
      universe = @parser.parse( :string => xml_ok )
    end    
    assert_equal 2, universe.nodes.size
  end
end