=begin
**
** commandline.rb
** 03/JUN/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Qt custom widget that behaves as a standard command line. It keeps a 
** buffer of commands that can be accessed by pressing Up and Down keys.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'Qt4' if $0 == __FILE__

module Ui
  module Qt4
    module Widgets
    
      # CommandLine is a custom Qt widget that behaves as a *standard* command 
      # line. It keeps a buffer of typed in commands that can be accessed by 
      # means of the +Up+ and +Down+ keys.
      # 
      # When the widget signals a +returnPressed+ the last command inserted by the
      # user is accessible using the last_command method.
      class CommandLine < Qt::LineEdit
        slots 'clear_history()'
      
        # Create the widget and initialize the internal data structures that will hold the _command_ _history_.
        def initialize(parent=nil)
          super(parent)
          #initialize internal history buffer
          @history = []
          @pointer = 0
          @up = true
        end
      
        #----------------------------------------------------- private methods
        private
        
        # Outputs the current command history buffer to $stdout
        def print_history(label)
          puts "-----#{label}-----"
          @history.each_index do |i|
            if (i == @pointer)
              puts "* #{@history[i]}"
            else
              puts "#{@history[i]}"
            end
          end
        end
      
        #----------------------------------------------------- public methods
        public
        #override some event handlers
        def keyPressEvent(event)
          #print_history('in') if (event.key == Qt::Key_Up || event.key == Qt::Key_Down || event.key == Qt::Key_Return)
          #p 'beb'
          case event.key 
            when Qt::Key_Up:
              if ((@history.size > 0) && (@pointer >= 0) )
                if (@pointer == @history.size)
                  @history << self.text 
                end
                @pointer -= 1 if @pointer > 0
                self.text = @history[@pointer]
              end
              return
      
            when Qt::Key_Down:
              if ((@history.size > 0) && (@pointer < @history.size) )
                @pointer += 1 if @pointer < @history.size - 1
                self.text = @history[@pointer]
              end
              return
      
            #add a new element to the local @history
            when Qt::Key_Return:
      
              #keep an eye on the last entry to avoid empty entries in the list
              if ((@history.size > 0) && (@history.last.strip.size == 0) )
                @history.pop
              end
      
              if self.text.strip.size > 0
                @history << self.text
                @pointer = @history.size
                self.clear
              end
          end
      
          #print_history('out') if (event.key == Qt::Key_Up || event.key == Qt::Key_Down || event.key == Qt::Key_Return)
          super
        end
      
        # Clears the internal command history buffer.
        def clear_history()
          @history.clear
        end
      
        # Returns the last command issued by the user. This method should be 
        # used when the parent window captures a +returnPressed+ event to
        # examine the las line inputted by the user.
        def last_command()
          @history.last
        end
      end
      
    end # module Widgets
  end #module Qt4
end #module Ui

if $0 == __FILE__
    a = Qt::Application.new(ARGV)
    w = Ui::Qt4::Widgets::CommandLine.new
    w.show
    a.exec
end
