=begin
**
** window.rb
** 01/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  Main wxWidgets application window
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'ui/wx/widgets'
require 'ui/wx/dialogs'

module Ui
  module WxWidgets

    class  DradisWindow < Wx::Frame
      consts = %w[
        ID_FileExit
        ID_ViewConsole
        ID_ViewHosts
        ID_TimerUpdates
      ]
      consts.each_with_index do | c, i |
        const_set(c, 2000 + i) 
      end
      attr_writer :controller
      attr_reader :consoleFrame
      
      #===================================================== private methods
      private      

      #----------------------------------------------------- setup the interface
      def setup_menu
        
        # application menu and menubar
        menu_bar = Wx::MenuBar.new

        file_menu = Wx::Menu.new
        file_menu.append(Wx::ID_EXIT, 'Exit')
        
        view_menu = Wx::Menu.new
        view_menu.append_check_item(ID_ViewConsole, 'Console')
        view_menu.append_check_item( ID_ViewHosts, 'Hosts')

        menu_bar.append(file_menu, 'File')
        menu_bar.append(view_menu, 'View')
        
        menu_bar.check(ID_ViewConsole, true)
        menu_bar.check(ID_ViewHosts, true)
        set_menu_bar(menu_bar)    
      end

      def setup_events
        # associate menus with their action methods
        # window menus
        evt_menu(Wx::ID_EXIT) { |e| on_exit }
        evt_menu(ID_ViewConsole) { |e| on_toggle_pane(:console)}
        evt_menu(ID_ViewHosts) { |e| on_toggle_pane(:hosts)}
                
        # advanced user interface
        evt_aui_pane_close { |e| on_pane_close(e) }
        
        # custom control event handling
        #-------------------------------------------------------------
        # console
        evt_new_command(@console.get_id) { |e| on_new_command(e) }
        
        # node tree
        evt_click_node(@hosts.get_id) { |e| on_click_node(e) }
        evt_right_click_node(@hosts.get_id) { |e| on_right_click_node(e) }
        evt_add_child_node(@hosts.get_id) { |e| on_add_child_node(e) }
        evt_rename_node(@hosts.get_id) {|e| on_rename_node(e) }
        evt_reassign_node(@hosts.get_id) { |e| on_reassign_node(e) }        
        
        # notes browser
        evt_add_note(@notes) { |e| on_add_note(e) }
        evt_edit_note(@notes) { |e| on_edit_note(e) }
        
        evt_properties_apply(@properties) { |e| on_properties_apply(e) }
        #-------------------------------------------------------------
        
        # setup periodic update timer
        @timer_updates = Wx::Timer.new(self, ID_TimerUpdates)
        @timer_updates.start( $config.get_option(:ui_refresh).to_i )
        evt_timer(ID_TimerUpdates) { |e| on_timeout_updates }
      end

      def setup_panes
        
        # Bottom panel: the console
        @console = Widgets::Console.new(self)
        pi_c = Wx::AuiPaneInfo.new
        pi_c.set_name('console_pane').set_caption('Console')
        pi_c.bottom.set_layer(1).set_position(1)
        @manager.add_pane(@console, pi_c)

        # Left panel: the nodes tree
        @hosts = Widgets::HostsBrowser.new(self)
        pi_h = Wx::AuiPaneInfo.new
        pi_h.set_min_size( Wx::Size.new(250, 250) )
        pi_h.set_name('hosts_pane').set_caption('Hosts')
        pi_h.left.set_layer(1).set_position(1)
        @manager.add_pane(@hosts, pi_h)

        # Main panel: The NoteBook, one tab for notes, one for properties,
        #   and in the future one for attachments
        notebook_style = Wx::AUI_NB_BOTTOM|
                          Wx::AUI_NB_SCROLL_BUTTONS|
                          Wx::AUI_NB_TAB_MOVE|
                          Wx::NO_BORDER
        # create the notebook off-window to avoid flicker
        client_size = get_client_size
        
        @notebook = Wx::AuiNotebook.new( self, 
                      Wx::ID_ANY,
                      Wx::Point.new(client_size.x, client_size.y),
                      Wx::Size.new(430, 200),
                      notebook_style)
                      
        # Page 1: notes
        @notes = Widgets::NotesBrowser.new(@notebook)
        @notebook.add_page( @notes, 'notes', false, Wx::NULL_BITMAP)

        # Page 2: properties
        @properties = Widgets::PropertiesPane.new(@notebook)
        @notebook.add_page( @properties, 'properties', false, Wx::NULL_BITMAP)

        pi_nb = Wx::AuiPaneInfo.new
        pi_nb.set_name('notebook_content').center_pane.hide
        @manager.add_pane(@notebook, pi_nb)
        @manager.get_pane('notebook_content').show
        
      end
      
      #----------------------------------------------------- menu actions
      def on_exit
        close(true)
      end
      
      def on_toggle_pane(panel_key)
        pane = nil
        case panel_key
        when :console
          pane = @manager.get_pane(@console)
        when :hosts
          pane = @manager.get_pane(@hosts)
        end
        pane.is_shown ? pane.hide : pane.show
        @manager.update
      end
      

      
      #----------------------------------------------------- advanced user interface
      def on_pane_close(event)
        id = nil
        case event.get_pane.get_name
        when 'console_pane'
          id = ID_ViewConsole
        when 'hosts_pane'
          id = ID_ViewHosts
        end
        get_menu_bar.find_item(id).check(false)
      end
      
      
      #----------------------------------------------------- custom controls
      def on_new_command(event)
        line = event.line
        output = @controller.request_service(:parse_command, line)
        @console.append(output.to_s)        
      end
      

      def on_click_node(event)
        $logger.debug { "on_click_node| #{event.node.id} has been clicked. #{event.node.label}" }
        #check if something was modified in the PropertiesPane and ask
        #the user that to do about it
        #if @properties.modified?
        #  dialog = Wx::MessageDialog.new(nil, 
        #              "Something was modified in the Properties window. If you move away, changes will be lost. Do you want to discard the changes?",
        #              'Information NOT saved', 
        #              Wx::NO_DEFAULT|Wx::YES_NO|Wx::ICON_QUESTION)
        #
        #  case dialog.show_modal()
        #  when Wx::ID_NO
        #    event.skip
        #    return
        #  end
        #end
        # TODO: we may need to pass the full object in the future, let's see
        @notes.show_notes(event.node.id, event.node.notes)
        @properties.show_properties(event.node.id, event.node.properties)        
      end
      
      def on_add_child(event)
        universe_info = @addchildinfo[event.get_id]
        
        @controller.request_service(:multiverse_create_node, 
                                    @node_popup.id,
                                    universe_info[:universe], 
                                    universe_info[:type])
      end
      
      def on_fire_populator(event)
        populator_info = @populatorinfo[event.get_id]
        $logger.debug{ "on_fire_populator reached" }
        $logger.debug{ populator_info }
        modified_properties = populator_info.invoke( 
                                :node_id => @node_popup.id, 
                                :properties => @node_popup.properties,
                                :controller => @controller )
        modified_properties.each do |prop|
          begin
            @properties.update_property(prop)
          rescue
            notify :exception, nil, $!
          end
        end
      end
      
      def on_right_click_node(event)
        # TODO: maybe we do not need to do this again and again
        @node_popup = event.get_client_data
        # FIXME: this is a hack :(
        @addchildinfo = {}
        @populatorinfo = {}
        
        menu = Wx::Menu.new
        
        # add child nodes
        addchild = Wx::Menu.new
        uid = 3000
        multiverse = @controller.request_service(:multiverse_universe_list)
        multiverse.each do |universe|
          next if universe.nodes.size.zero?
          universemenu = Wx::Menu.new
          universe.nodes.each do |node|
            uid += 1
            universemenu.append uid, node[:name]
            @addchildinfo[uid] = { :universe => universe.meta[:uid], :type => node[:type]}
            evt_menu(uid) { |e| on_add_child(e) }
          end
          addchild.append_menu Wx::ID_ANY, universe.meta[:name], universemenu
        end
        
        # populators menu
        populators = Wx::Menu.new
        p @node_popup.properties
        @node_popup.properties.each do |prop|
          populator = @controller.request_service(:multiverse_populator_for, 
                                                    @node_popup.universe_uid,
                                                    @node_popup.node_type, 
                                                    prop.key)
          if populator
            uid += 1
            populators.append uid, prop.name
            @populatorinfo[uid] = populator
            evt_menu(uid) { |e| on_fire_populator(e) }
          end
        end
        
        menu.append_menu Wx::ID_ANY, 'add child', addchild
        if populators.get_menu_item_count() > 0
          menu.append_menu Wx::ID_ANY, 'populators', populators 
        end
        popup_menu menu
      end
      
      def on_add_child_node(event)
        $logger.debug { "on_add_child_node| " }
        node = event.node
        @controller.request_service(:model_add_node, node.parent_id, node.type_id, node.label)
      end
      
      def on_rename_node(event)
        $logger.debug { "on_rename_node| #{event.node.id} has been relabeled to #{event.node.label}" }
        @controller.request_service(:model_update_node, 
            event.node.id, 
            event.node.parent_id,
            event.node.type_id,
            event.node.label)
      end
      
      def on_reassign_node(event)
        $logger.debug { "on_reassign_node| #{event.node.id} has been reassigned to #{event.node.parent_id}" }
        @controller.request_service(:model_update_node,
            event.node.id, 
            event.node.parent_id,
            event.node.type_id,
            event.node.label)
        @prev_selection = nil
      end
      
      
      def on_add_note(event)
        # To prevent our event from being invalidated, lets cache the values
        # we need
        node_id = event.node_id

        dialog = Dialogs::NoteDialog.new
        dialog.categories = @controller.request_service(:model_categories_list)
        if dialog.show_modal == Wx::ID_OK
          success = @controller.request_service(:model_add_note,
            node_id, 
            dialog.author, 
            dialog.category_id, 
            dialog.text)
          if success
            @notes.invalidate!
          end
        end
        dialog.destroy
      end
      def on_edit_note(event)
        # To prevent our event from being invalidated, lets cache the values
        # we need
        node_id = event.node_id
        note_id = event.note_id
        
        note = @controller.request_service(:model_find_note, note_id)
        
        dialog = Dialogs::NoteDialog.new
        dialog.categories = @controller.request_service(:model_categories_list)
        dialog.edit_note(note)
        
        if dialog.show_modal == Wx::ID_OK
          if (( dialog.author != note.author ) |
            ( dialog.category_id != note.category_id ) |
            ( dialog.text != note.text )
            )
            success = @controller.request_service(:model_update_note,
              node_id,
              note_id, 
              dialog.author, 
              dialog.category_id, 
              dialog.text)
          
            if success
              @notes.invalidate!
            end
          end          
        end
        dialog.destroy
      end
      
      def on_properties_apply(event)
        $logger.debug{ 'on_properties_apply' }
        @controller.request_service( :multiverse_save_node_properties, 
                                      event.get_client_data(),
                                      @properties.get_properties())
      end
      #----------------------------------------------------- other events
      def on_timeout_updates
        if (@controller.request_service(:model_knowledge_new_revision?))
          Wx::WindowDisabler.disable(self) do
            info = Wx::BusyInfo.busy("Working, please wait...", self) do
              # refresh the model
              @controller.request_service(:model_knowledge_refresh)
              
              # yields control to pending messages in the windowing system. 
              Wx::get_app.yield()
              
              # update the view
              update_view
            end
          end
          @console.append( "New revision downloaded from the server at #{DateTime.now.strftime('%H:%M:%S')}." )
        end
      end
      
      def update_view
        @prev_selection = nil
        @hosts.update_view(@controller.request_service(:model_knowledge_get))
        @notes.update_view(@controller.request_service(:model_knowledge_get))
        @properties.update_view()
        @notebook.set_selection(0)
      end
      #===================================================== public methods
      public
      def initialize(*args)
        super
        @manager = Wx::AuiManager.new
        @manager.set_managed_window(self)
        setup_menu()
        #create_status_bar()
        set_min_size( Wx::Size.new(400,300) )
        setup_panes()
        setup_events()
        @manager.update()
      end         
      
      def notify(event, *args)
        case event
          when :exception
            # TODO: handle each exception properly
            @console.append( args[1] )  
        end
      end

      # Receive the application quit request from the controller, display
      # a goodbye message and close the frame.
      def teardown()
        @console.append( 'closing dradis interface...' )
        self.close
      end
    end
  end
end
